div.styledLogo.tab {
    border: none;
    background: transparent;
    outline: none;
    cursor: pointer;
    text-align: left;
    box-sizing: none;
    padding: 20px 30px;
}

div.styledButton.tab {
    border: none;
    background: transparent;
    outline: none;
    cursor: pointer;
    transition: background-color, color 200ms;
    text-align: left;
    box-sizing: none;
    font-size: 16px;
    font-family: var(--menuLinkFont);
    font-weight: var(--menuLinkFontWeight);
    font-style: var(--menuLinkFontStyle);
    color: var(--menuLinkColor);
    text-decoration: none;
    padding: 20px 30px;
    border-bottom: 1px solid white;
}

div.styledButton.tab:hover {
    color: var(--menuLinkHoverColor);
    background-color: var(--menuLinkBackgroundHover);
}

div.selectedButton.tab--selected {
    color: var(--menuLinkColorSelected);
    background-color: var(--menuLinkBackgroundSelected);
}

div.pageWrapper.tab-content {
    display: flex;
    flex-direction: column;
    width: 100%;
    margin: 20px;

}

.layoutWrapper {
    width: 100%;
}

.wrapper {
    display: flex;
    height: 100%;
    width: 100%;
}

.logoWrapper {
    margin: 12px 20px 20px 18px;
    margin: 20px 30px;
    display: flex;
    align-items: center;
}

/* Needed to override border for the last tab */
#last_page {border: none!important;} /* csslint allow: known-properties, important */
#logo {border: none!important;} /* csslint allow: known-properties, important */