<html>
<head>
<meta charset="UTF-8"> 
<link rel="stylesheet" href="./assets/webviz_doc.css">

<!-- Also update package-lock.json with highlight.js version number: -->
<script src="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/9.14.2/highlight.min.js" integrity="sha256-sNPiigbfSkqUzMc5rgrdztLnneCMAp6W9wetJUZu9Zw=" crossorigin="anonymous"></script>
<script src="https://cdnjs.cloudflare.com/ajax/libs/highlight.js/9.14.2/languages/yaml.min.js" integrity="sha256-GuL67zsJWO0bMHHaASSslF37TSgnQF5fuL2n80/HpoU=" crossorigin="anonymous"></script>

<script>hljs.initHighlightingOnLoad();</script>

</head>

<body>

<div class='container'>

<h1>Webviz configuration guide</h1>

<h2>Fundamental configuration</h2>

A configuration consists of some mandatory properties (title, username,
password) and one or more pages. Each page has a title, and potentially some
content. The content can be one or more items.

Containers represent predefined content, which takes one or more arguments.
A list and description of all available different containers is listed below.

Content which is not containers are interpreted as text paragraphs.

A basic example configuration is shown below.

<pre><code class='yaml'>{{ basic_example }}</code></pre>

<h2>Container documentation</h2>

{%- for package in packages.values() %}

{{- package['doc'] -}}

<div style='margin-left: 40px;'>

{%- for container in package['containers'] %}

{{ container['doc'] }}
<pre><code class='yaml'>  - container: {{ container['name'] }}
    {%- for arg in container['args'] %}
    {{ arg }}: {{ container['values'][arg] }}
    {%- endfor %}
</code></pre>
<hr>
{% endfor %}
</div>
{%- endfor %}

</div>
</div>

</body>

</html>
